package ch.hslu.swk.g03.logger.server;

import java.io.BufferedWriter;
import java.io.File;
import java.io.FileWriter;
import java.io.IOException;
import java.util.ArrayList;
import java.util.Iterator;

import ch.hslu.swk.g03.logger.common.LogMessage;

/**
 * LogMessage DB (File). http://tools.ietf.org/html/rfc4180
 *
 * @author Felix Rohrer <felix.rohrer@stud.hslu.ch>
 */
public final class LoggerDB {

    private static LoggerDB instance;
    private String logFileName;
    private final ArrayList<LogMessage> logMessageDB;
    private File logFile;
    private boolean logfileOpen;
    private BufferedWriter logWriter;

    /**
     * Default Constructor.
     */
    private LoggerDB() {
	logMessageDB = new ArrayList<LogMessage>();
    }

    /**
     * LoggerDB Singleton.
     *
     * @return LoggerDB Instance
     */
    public static LoggerDB getInstance() {
	synchronized (LoggerDB.class) {
	    if (LoggerDB.instance == null) {
		try {
		    LoggerDB.instance = new LoggerDB();
		} catch (Exception e) {
		    System.err.println("[ERROR:LoggerDB] getInstance()");
		    System.err.println(e.getStackTrace());
		}
	    }
	    return LoggerDB.instance;
	}
    }

    /**
     * Sets the FileName.
     *
     * @param fileName
     *            new Logfile Name
     */
    public void setLogfileName(final String fileName) {
	synchronized (LoggerDB.class) {
	    this.logFileName = fileName;
	}
    }

    /**
     * Opens the Logfile.
     *
     * @return true if open logfile was successfully
     * @throws IOException
     *             if logfile cannot be opened or created
     */
    public boolean openLogfile() throws IOException {
	synchronized (LoggerDB.class) {
	    boolean writeHeader = false;
	    if (!logfileOpen) {
		logFile = new File(this.logFileName);

		if (!logFile.exists()) { // if file doesnt exists, create it
		    logFile.createNewFile();
		    writeHeader = true;
		    System.out.println("[INFO:LoggerDB] new Logfile createt: " + logFile.getAbsoluteFile());
		}
		logWriter = new BufferedWriter(new FileWriter(logFile.getAbsoluteFile(), true)); // true
		// =
		// append
		if (writeHeader) {
		    logWriter.write("\"ID\",\"Timestamp\",\"Loglevel\",\"Computer\",\"Source\",\"Message\"\n");
		    logWriter.flush();
		}
		logfileOpen = true;
	    }
	    return logfileOpen;
	}
    }

    /**
     * Close the logfile.
     *
     * @return true if successfully closed the file
     * @throws IOException
     *             if logfile cannot be closed
     */
    public boolean closeLogfile() throws IOException {
	synchronized (LoggerDB.class) {
	    if (logfileOpen) {
		logWriter.close();
		logfileOpen = false;
	    }
	    return !logfileOpen;
	}
    }

    /**
     * Add a new Message to the Logfile.
     *
     * @param newMsg
     *            new LogMessage
     */
    public void addMessage(final LogMessage newMsg) {
	synchronized (LoggerDB.class) {
	    logMessageDB.add(newMsg);
	    System.out.println("[INFO:LoggerDB] Save: " + newMsg.toString());

	    if (logfileOpen) {
		try {
		    logWriter.write(newMsg.toString() + "\n");
		    logWriter.flush();
		} catch (IOException e) {
		    System.err.println("[ERROR:LoggerDB] could not write message into logfile!");
		    System.err.println(e.getStackTrace());
		}
	    }
	}
    }

    /**
     * Return a ArrayList with all LogMessages since start.
     *
     * @return LogMessage ArrayList
     */
    public ArrayList<LogMessage> getAllMessages() {
	return this.logMessageDB;
    }

    /**
     * Print all LogMessages since start.
     */
    public void printAllMessages() {
	LogMessage msg;
	System.out.println("[INFO:LoggerDB] Messages since LoggerServer start: " + this.logMessageDB.size());
	final Iterator<LogMessage> iterator = this.logMessageDB.iterator();
	while (iterator.hasNext()) {
	    msg = iterator.next();
	    System.out.println(" --> " + msg.getId() + ": " + msg.getMessage());
	}
    }

    /**
     * Return the LogMessage at index.
     *
     * @param index
     *            LogMessage Index
     * @return LogMessage
     */
    public LogMessage getMessage(final int index) {
	if (index >= 0 && index < logMessageDB.size()) {
	    return logMessageDB.get(index);
	} else {
	    return null;
	}
    }

    /**
     * Return the Number of Messages inside the current Log.
     * @return Number of LogMessages
     */
    public int getMessageCount() {
	return logMessageDB.size();
    }

}
